/*
*******************************************************************************************************************
*									TONE GENERATION DEMO BY SOFTWARE TIMER
* Description:  A demo program for AT89S52-gLCD-STK1 board with 3.3V regulator & 89C51RB2-IM mcu
*				to blink an LED at ~10Hz by software delay
*				&& using timer 2 to generate a single tone by the Buzzer connected to Pin P1.0.
*				Timing is done by software delay loop.
* File:			..\Buzzer\main.c
* Software:		Keil C (C51) compiler, Eval version 7.09 under uVision2 IDE v2.40
* Programmer: 	John Leung, TechToys Co. (www.TechToys.com.hk)
* Hardware:		AT89S8253-cLCD-STK1, with Atmel AT89C51RB2 running at 3.3V Vdd, 11.0592 MHz crystal
* Date:			29th Dec 2006
* Version:		1.0
* Programmer:	John Leung
*												 ALL RIGHTS RESERVED
*******************************************************************************************************************
*/

#include <REG51RB2.h>

#define ON 	0
#define OFF	1

/*
*******************************************************************************************************************
*												HARDWARE DEFINITION
*******************************************************************************************************************
*/
sbit LED 	= P2^0;
sbit BUZZER = P1^0;

/*
*******************************************************************************************************************
*												FUNCTION PROTOTYPE
*******************************************************************************************************************
*/
void Tmr0Init();
void ToneGen(unsigned int tone, unsigned int duration);
void DelayMs(unsigned int msec);


/*
*******************************************************************************************************************
*													ISR SECTION
*******************************************************************************************************************
*/

/*
*******************************************************************************************************************
*										Timer 0 interrupt service routine
* Function:	This function toggles LED pin and reload Timer 0 for a 10ms interrupt. A software counter 'cnt' is 
*			used to control the rate of toggle (5x10ms = 50ms)
* Argument:	none
* Return:	none
* Note:	
*******************************************************************************************************************
*/
Tmr0() interrupt 1
{
	unsigned char cnt;
	cnt++;
	if(cnt==5)	{LED = ~LED; cnt=0;}
	TH0 =	0xDC;			//10ms hardware delay, 11.0592MHz => 1 machine cycle at 1.085usec
	TL0 =	0x00;			//Number of cycles to delay 10ms = 10000/1.085 = 9216
							//T0 in 16-bit mode => T0 preload with a value (65536-9216)=56,320, i.e. 0xDC00
}							


/*
*******************************************************************************************************************
*												FUNCTION DECLARATION
*******************************************************************************************************************
*/

/*
*******************************************************************************************************************
*										Timer 0 initialization routine
* Function:	This function initializes Timer 0 to 16-bit manual reload mode while leaving Timer 1 parameters
*			unchanged. Timer 0 interrupt enabled and started here.
* Argument:	none
* Return:	none
* Note:	
*******************************************************************************************************************
*/
void Tmr0Init(void)
{

	TMOD=	TMOD&0xF0;		//reset TIMER0 configuration
	TMOD=	TMOD|0x01;		//leave TIMER1 configuration unchanged and set T0 to 16-bit timer mode

	TH0 =	0xDC;			//10ms hardware delay, 11.0592MHz => 1 machine cycle at 1.085usec
	TL0 =	0x00;			//Number of cycles to delay 10ms = 10000/1.085 = 9216
							//T0 in 16-bit mode => T0 preload with a value (65536-9216)=56,320, i.e. 0xDC00
	ET0	=	1;				//enable T0 interrupt
	TR0 =	1;				//start timer 0
}


/*
*******************************************************************************************************************
*											TONE GENERATION ROUTINE
* Function:	This function generates a single tone of a certain frequency 'tone' in Hz, for a given period of time
*			'duration' in msec by using Timer 2 in programmable clock out mode
* Argument:	'tone'		frequency of the tone to generate, in Hz
*			'duration'	duration of tone generate in millisec.
* Return:	none
*
* Note:		*** Please note that there is a limit in the frequency range. We are using a crystal frequency of
*				11.0592MHz. From equal Clock-out Freq= Oscillator Freq / (4*(65536-RCAP2H,RCAP2L)).
*				Thus, max Clock-out Freq = 2.7648MHz; min Clock-out Freq = 42 Hz.
*				Human audible range is around 20Hz - 20kHz, and the frequency response of piezo sounder has 
*				a limited range, too. We just cannot make infinite pitch from our hardware! ***
*******************************************************************************************************************
*/
void ToneGen(unsigned int tone, unsigned int duration)
{
	unsigned long tmp;
	if(tone!=0)
	{
	T2CON = 0x00;			// reset T2CON, the Timer 2 control register. Important for stopping Timer 2 by TR2=0
 
	T2MOD = T2MOD&0xFE;
	T2MOD = T2MOD|0x02;		// sets T2OE=1; DCEN=0; for a clock out mode, no interrupt generated in this mode

	tmp = (11059200UL>>2)/tone;
	tmp = 65536UL - tmp;	
	RCAP2H = (unsigned char) (tmp>>8);
	RCAP2L = (unsigned char) (tmp&0x000000FF);
	TH2 = RCAP2H;
	TL2 = RCAP2L;

	TR2 = 1;				//Start Timer 2 for clock out
	DelayMs(duration);		//software timer here
	TR2 = 0;
	}
	BUZZER = OFF;			//make sure BUZZER is off on exit
}


/*
*******************************************************************************************************************
*											MILLI-SECOND SOFTWARE DELAY ROUTINE
* Function:	This function makes software delay in milli-sec, adjusted for a crystal freq. 11.0592MHz for AT89S52
* Argument:	'msec'		amount of delay in milli-sec
* Return:	none
*
* Note:		
*******************************************************************************************************************
*/
void DelayMs(unsigned int msec)
{
	unsigned int x,y;

	for(x=0; x<=msec;x++)
		{
			for(y=0;y<=110;y++);
		}
}

/*
*******************************************************************************************************************
*														MAIN
*******************************************************************************************************************
*/

void main(void)
{
	Tmr0Init();
	EA = 1;					//enable global interrupt flag

	for(;;)
	{	
	ToneGen(262, 1000);	//Do
	ToneGen(294, 1000);	//Re
	ToneGen(330, 1000);	//Mi
	ToneGen(349, 1000);	//Fa
	ToneGen(392, 1000);	//So
	ToneGen(440, 1000);	//La
	ToneGen(494, 1000); //Si
	}

}
